#ifndef H_OOFRW
#define H_OOFRW
// COPYRIGHT 1994 A.D. Software, All rights reserved

// report-writer layer of OOFILE database

// NOTE inline definitions included at end of this header file

#include "oof2.hpp"
#include "oofview.hpp"

class dbRepColWidths : public OOF_ExpandableLongArray {
public:
// construction
	dbRepColWidths (unsigned int defColWidth=20) :
							OOF_ExpandableLongArray(defColWidth) {};
	// use default copy ctor
	dbRepColWidths& operator<<(unsigned int);
};


class dbRepPageBreak : public OOF_ExpandableLongArray {
public:
// construction
	dbRepPageBreak (unsigned int breakPos=0xFFFF) :
							OOF_ExpandableLongArray(breakPos) {};
	private:
		dbRepPageBreak(const dbRepPageBreak&) { assert(0); };
		void operator=(const dbRepPageBreak&) { assert(0); };
};


class dbRepOverRun : public dbClass{
	public:
		dbRepOverRun() :
			OverRun(0),
			DeleteMe(0) {};
		~dbRepOverRun() {};
	private:
		dbRepOverRun(const dbRepOverRun&) { assert(0); };
		void operator=(const dbRepOverRun&) { assert(0); };

public:		
		char* OverRun;
		char* DeleteMe;
};


class dbRepLine : public dbClass {
	public:
		dbRepLine() :
			mLine(0) {};
		~dbRepLine();
	private:
		dbRepLine(const dbRepLine&) { assert(0); };
		void operator=(const dbRepLine&) { assert(0); };
	
	public:
		
		void prepare();
		void clear();
		void drawNCharsAt(unsigned int hPos, char* theChars, unsigned int len);
		void fillNCharsAt(unsigned int hPos, char theChar, unsigned int len);
		void drawToStream(unsigned int hPos, unsigned int len, ostream& os);
		
		char *mLine;		// owned									// for drawing onto
		static unsigned int mWidth;		// size of horizontal line - given to us by dbRepPage

};


class dbRepSizer : public dbClass {
public:
// construction
	dbRepSizer (const OOF_String& reportTitle, 
							unsigned int pageHeight=70, 
							unsigned int pageWidth=80, 
							unsigned int leftMargin=6, 
							unsigned int rightMargin=6, 
							unsigned int topMargin=1, 
							unsigned int bottomMargin=1, 
							unsigned int colSepWidth=2, 
							unsigned int blockVertSep=1
							) :
							mReportTitle(reportTitle),
							mPageHeight(pageHeight),
							mPageWidth(pageWidth),
							mLeftMargin(leftMargin),
							mRightMargin(rightMargin),
							mTopMargin(topMargin),
							mBottomMargin(bottomMargin),
							mColSepWidth(colSepWidth),
							mBlockVertSep(blockVertSep) {};
	
	dbRepSizer (const char * reportTitle="", 
							unsigned int pageHeight=70, 
							unsigned int pageWidth=80, 
							unsigned int leftMargin=6, 
							unsigned int rightMargin=6, 
							unsigned int topMargin=1, 
							unsigned int bottomMargin=1, 
							unsigned int colSepWidth=2, 
							unsigned int blockVertSep=1
							) :
							mReportTitle(reportTitle),
							mPageHeight(pageHeight),
							mPageWidth(pageWidth),
							mLeftMargin(leftMargin),
							mRightMargin(rightMargin),
							mTopMargin(topMargin),
							mBottomMargin(bottomMargin),
							mColSepWidth(colSepWidth),
							mBlockVertSep(blockVertSep)  {};
	// use default copy ctor

// setters for use in simulating keyword arguments
// eg dbRep( dbRepSizer("Andy's Report").pageHeight(80).DefColWidth(12) );
	
	dbRepSizer& title(const OOF_String&);
	dbRepSizer& title(const char*);
	dbRepSizer& pageHeight(unsigned int);
	dbRepSizer& pageWidth(unsigned int);
	dbRepSizer& leftMargin(unsigned int);
	dbRepSizer& rightMargin(unsigned int); 
	dbRepSizer& topMargin(unsigned int); 
	dbRepSizer& bottomMargin(unsigned int); 
	dbRepSizer& colSepWidth(unsigned int);
	dbRepSizer& blockVertSep(unsigned int);

// presentation	
	
	OOF_String mReportTitle;
	unsigned int mPageHeight, mPageWidth, mLeftMargin, mRightMargin, mTopMargin, mBottomMargin;
	unsigned int mColSepWidth;		// whitespace between columns
	unsigned int mBlockVertSep;		// whitespace between records on columnar and blocks on pagewise

};


class dbRepPage : public dbClass {
	public:
		dbRepPage();
		~dbRepPage();
	private:
		dbRepPage(const dbRepPage&) { assert(0); };
		void operator=(const dbRepPage&) { assert(0); };

	public:		
		void draw(dbRepSizer Sizer, ostream& os);
		void clearLines(unsigned int start, unsigned int end);
		void endPage(ostream& os);
		
		 dbRepLine* mPageMap;				// array of horizontal bands  	// owned
		 unsigned int	mWidth;				// width of valid lines
		 unsigned int mNumLines;			// number of prinatble lines
		 unsigned int mBodyStart;			// position of data area under header
		 									//   to save redrawing the header
		 unsigned int mNumPages;			// number of physical pages
		 unsigned int* mFieldPos;			// starting position of each field	// owned
		 									//   within each dbRepLine
		 dbRepPageBreak mPageBreak;			// starting position of each new page
		 									//   within each dbRepLine
		 unsigned int* mPageStart;			// starting Row on page below Header	// owned

};

class dbRep : public dbClass {
public:
	enum ReportStyles {pageWise, columnar};
// construction
	dbRep(const dbRepSizer& sizer, const dbView& fields, const ReportStyles style):
				mSizer(sizer),
				mFields(fields),
				mReportStyle(style),
				mBuilt(false) {};
	
	dbRep(const dbRepSizer& sizer, const dbRepColWidths colWidths, const dbView& fields, const ReportStyles style):
				mSizer(sizer),
				mFields(fields),
				mReportStyle(style),
				mColWidths(colWidths),
				mBuilt(false) {};
				
	// use default copy ctor
	virtual ~dbRep() {};
	
	void setStyle(const ReportStyles);

// presentation	
	virtual void formatForCharStream() = 0;
	virtual void draw(ostream&) = 0;
	
	void extract(ostream&);
	char *copyStr(char *theString);

protected:	
	dbRepSizer		mSizer;
	dbView 			mFields;
	ReportStyles 	mReportStyle;
	dbRepColWidths	mColWidths;
	bool			mBuilt;
};

class dbRepChar : public dbRep{
public:
// construction
	dbRepChar(const dbRepSizer& sizer, const dbView& fields, const ReportStyles style=columnar):
				dbRep(sizer, fields, style),
				mPage() {};
	
	dbRepChar(const dbRepSizer& sizer, const dbRepColWidths colWidths, const dbView& fields, const ReportStyles style=columnar):
				dbRep(sizer, colWidths, fields, style),
				mPage() {};
	
	// use default copy ctor
	virtual ~dbRepChar() {};
	
// presentation	
	virtual void formatForCharStream();		// may be others later
	virtual void draw(ostream&);

	unsigned int drawWrappedChars(unsigned int line,unsigned int hPos,unsigned int width,char **theString);
	void drawColumnar(ostream&);
	void drawPageWise(ostream&);
	void drawHeader(ostream&);
	
private:
	dbRepPage mPage;
};

// include inline definitions
#include "oofrw.inl"

#endif